import 'package:clean_android_design/clean_android_design.dart';
import 'package:clean_android_design/src/settings/settings.dart';
import 'package:cloud_firestore/cloud_firestore.dart';
import 'package:flutter/material.dart';
import 'package:provider/provider.dart';

class SelectableListFieldWidget<T extends ReadOnlyDatabaseItem>
    extends StatefulWidget {
  final bool isFirst;
  final bool isLast;

  final void Function(List<DocumentReference> selectedItemRefs)
      saveSelectedValues;
  final List<DocumentReference> Function() getSelectedValues;

  final bool Function(T item) selectableItemsLambda;
  final ISelectableListItem Function(BuildContext context, T item) itemBuilder;
  final Comparable Function(T item) sortProperty;

  final bool isEditable;

  final Widget noItemWidget;
  final Widget noItemListItem;

  final Widget topWidget;
  final Widget title;

  final ForceUpdateKey activateKeyOnChange;
  final ItemSectionExtendedController controller;

  const SelectableListFieldWidget({
    Key key,
    this.saveSelectedValues,
    @required this.getSelectedValues,
    this.selectableItemsLambda,
    this.itemBuilder,
    this.noItemWidget,
    this.noItemListItem,
    this.sortProperty,
    this.topWidget,
    this.title,
    this.activateKeyOnChange,
    @required this.isEditable,
    this.controller,
  })  : isFirst = false,
        isLast = false,
        assert(getSelectedValues != null),
        assert(!isEditable || saveSelectedValues != null),
        assert(noItemListItem == null || noItemWidget == null),
        super(key: key);

  @override
  _SelectableListFieldWidgetState<T> createState() =>
      _SelectableListFieldWidgetState<T>();
}

class _SelectableListFieldWidgetState<T extends ReadOnlyDatabaseItem>
    extends State<SelectableListFieldWidget<T>> {
  List<T> selectableItems;

  @override
  Widget build(BuildContext context) {
    if (selectableItems == null) {
      selectableItems = Provider.of<List<T>>(context)
          .where((item) => widget.selectableItemsLambda?.call(item) ?? true)
          .toList();

      if (widget.sortProperty != null)
        selectableItems.sort((i1, i2) =>
            widget.sortProperty(i1).compareTo(widget.sortProperty(i2)));
    }

    return _getDefautFieldWidget(widget.isEditable);
  }

  Widget _getDefautFieldWidget(bool isEditable) {
    var items = isEditable
        ? selectableItems
        : selectableItems
            .where(
                (e) => widget.getSelectedValues().any((ref) => ref.id == e.id))
            .toList();

    if (items.length == 0 && widget.noItemListItem == null)
      return widget.noItemWidget ?? Container();

    return Column(
      children: <Widget>[
        widget.topWidget ?? Container(),
        items.length != 0
            ? ItemSectionWidget.builder(
                title: widget.title,
                controller: widget.controller,
                itemBuilder: (context, index) =>
                    widget.itemBuilder(context, items[index]).copyWithSelection(
                          isSelected: isEditable
                              ? widget
                                  .getSelectedValues()
                                  .any((ref) => ref.id == items[index].id)
                              : false,
                          onTapList: isEditable
                              ? () {
                                  _changeSelectionFor(items[index]);
                                  return false;
                                }
                              : null,
                        ),
                itemCount: items.length,
              )
            : (widget.noItemListItem == null
                ? Container()
                : ItemSectionWidget(
                    title: widget.title,
                    controller: widget.controller,
                    children: [widget.noItemListItem],
                  )),
      ],
    );
  }

  _changeSelectionFor(T item) {
    var selectedItems = widget.getSelectedValues();
    if (selectedItems.any((ref) => ref.id == item.id))
      selectedItems.removeWhere((ref) => ref.id == item.id);
    else
      selectedItems.add(CleanAndroidDesignSettings.firestore
          .doc("${item.getTypeName()}/${item.id}"));

    setState(() => widget.saveSelectedValues(selectedItems));
    widget.activateKeyOnChange?.update();
  }
}
